/*
 * drivers/storage/ata.h
 *
 * See drivers/storage/ata.c for copyright info.
 *
 */
#ifndef __LINUX_ATA_H__
#define __LINUX_ATA_H__

#include "storage.h"

enum {
	ATA_STD_NCHAN = 2,	/* num channels on a standard ATA device */

	/* ATA_CMD_FL_HAVE_xxx indicates the ATA register value is
	 * present and should be written to the hardware.  If a flag
	 * is not present, code should assume the register's value
	 * as stored in struct ata_cmd is undefined, and it should
	 * not be written to the hardware.
	 */
 						  /* register name */
	ATA_CMD_FL_HAVE_DATA		= (1 << 0), /* data */
	ATA_CMD_FL_HAVE_NSECTOR		= (1 << 1), /* nsector */
	ATA_CMD_FL_HAVE_LBALOW		= (1 << 2), /* lba low */
	ATA_CMD_FL_HAVE_LBAMID		= (1 << 3), /* lba mid */
	ATA_CMD_FL_HAVE_LBAHIGH		= (1 << 4), /* lba high */
	ATA_CMD_FL_HAVE_FEATURE		= (1 << 5), /* feature */

	ATA_CMD_FL_HAVE_HOB_DATA	= (1 << 8), /* HOB data */
	ATA_CMD_FL_HAVE_HOB_NSECTOR	= (1 << 9), /* HOB nsector */
	ATA_CMD_FL_HAVE_HOB_LBALOW	= (1 << 10), /* HOB LBA low */
	ATA_CMD_FL_HAVE_HOB_LBAMID	= (1 << 11), /* HOB LBA mid */
	ATA_CMD_FL_HAVE_HOB_LBAHIGH	= (1 << 12), /* HOB LBA high */
	ATA_CMD_FL_HAVE_HOB_FEATURE	= (1 << 13), /* HOB feature */

	ATA_CMD_FL_HAVE_DEVCTL		= (1 << 16), /* device control */

	/* indices into ata_cmd_prep array */
	ATA_PREP_DEVDIAG		= 0,
	ATA_PREP_ID			= 1,
	ATA_PREP_ID_PACKET		= 2,
	
	/* ATA commands */
	ATA_CMD_EXEC_DEV_DIAG		= 0x90,
	ATA_CMD_ID			= 0xEC,
	ATA_CMD_ID_PACKET		= 0xA1,
	
	/* ATA status register bits */
	ATA_BUSY			= (1 << 7),
	ATA_DATA_RQ			= (1 << 3),
	ATA_ERR				= (1 << 0),

	/* ATA device control register bits */
	ATA_NO_INTR			= (1 << 1),
	ATA_HOB				= (1 << 7),
	ATA_SOFT_RESET			= (1 << 2),
	
	/* ATA device register bits */
	ATA_DEV1			= (1 << 4), /* 0==dev0, 1==dev1 */

	/* ATA error register bits */
	ATA_ABORTED_CMD			= (1 << 2),
	
	/* size of IDENTIFY [PACKET] DEVICE id page, in 16-bit words */
	ATA_ID_PAGE_SIZE		= 256,
	
	/* struct ata_channel flags */
	ATA_CHAN_FL_ENABLED		= (1 << 0),
	
	/* struct ata_device flags */
	ATA_DEV_FL_REMOVEABLE		= (1 << 7),
	ATA_DEV_FL_48BIT		= (1 << 0),
	ATA_DEV_FL_LBA			= (1 << 1),

	/* IDENTIFY [PACKET] DEVICE indices and bit definitions */
	ATA_ID_CONFIG			= 0,

	/* locking strategies */
	ATA_LOCK_HOST			= 1,
	ATA_LOCK_CHANNEL		= 2,
};

struct ata_id_page {
	u16	config;		/* lots of obsolete bit flags */
	u16	cyls;		/* Obsolete, "physical" cyls */
	u16	reserved2;	/* reserved (word 2) */
	u16	heads;		/* Obsolete, "physical" heads */
	u16	track_bytes;	/* unformatted bytes per track */
	u16	sector_bytes;	/* unformatted bytes per sector */
	u16	sectors;	/* Obsolete, "physical" sectors per track */
	u16	vendor0;	/* vendor unique */
	u16	vendor1;	/* vendor unique */
	u16	vendor2;	/* Retired vendor unique */
	u8	serial_no[20];	/* 0 = not_specified */
	u16	buf_type;	/* Retired */
	u16	buf_size;	/* Retired, 512 byte increments
				 * 0 = not_specified
				 */
	u16	ecc_bytes;	/* for r/w long cmds; 0 = not_specified */
	u8	fw_rev[8];	/* 0 = not_specified */
	u8	model[40];	/* 0 = not_specified */
	u8	max_multsect;	/* 0=not_implemented */
	u8	vendor3;	/* vendor unique */
	u16	dword_io;	/* 0=not_implemented; 1=implemented */
	u8	vendor4;	/* vendor unique */
	u8	capability;	/* (upper byte of word 49)
				 *  3:	IORDYsup
				 *  2:	IORDYsw
				 *  1:	LBA
				 *  0:	DMA
				 */
	u16	reserved50;	/* reserved (word 50) */
	u8	vendor5;	/* Obsolete, vendor unique */
	u8	tPIO;		/* Obsolete, 0=slow, 1=medium, 2=fast */
	u8	vendor6;	/* Obsolete, vendor unique */
	u8	tDMA;		/* Obsolete, 0=slow, 1=medium, 2=fast */
	u16	field_valid;	/* (word 53)
				 *  2:	ultra_ok	word  88
				 *  1:	eide_ok		words 64-70
				 *  0:	cur_ok		words 54-58
				 */
	u16	cur_cyls;	/* Obsolete, logical cylinders */
	u16	cur_heads;	/* Obsolete, l heads */
	u16	cur_sectors;	/* Obsolete, l sectors per track */
	u16	cur_capacity0;	/* Obsolete, l total sectors on drive */
	u16	cur_capacity1;	/* Obsolete, (2 words, misaligned int)     */
	u8	multsect;	/* current multiple sector count */
	u8	multsect_valid;	/* when (bit0==1) multsect is ok */
	u32	lba_capacity;	/* Obsolete, total number of sectors */
	u16	dma_1word;	/* Obsolete, single-word dma info */
	u16	dma_mword;	/* multiple-word dma info */
	u16  eide_pio_modes; /* bits 0:mode3 1:mode4 */
	u16  eide_dma_min;	/* min mword dma cycle time (ns) */
	u16  eide_dma_time;	/* recommended mword dma cycle time (ns) */
	u16  eide_pio;       /* min cycle time (ns), no IORDY  */
	u16  eide_pio_iordy; /* min cycle time (ns), with IORDY */
	u16	words69_70[2];	/* reserved words 69-70
				 * future command overlap and queuing
				 */
	/* HDIO_GET_IDENTITY currently returns only words 0 through 70 */
	u16	words71_74[4];	/* reserved words 71-74
				 * for IDENTIFY PACKET DEVICE command
				 */
	u16  queue_depth;	/* (word 75)
				 * 15:5	reserved
				 *  4:0	Maximum queue depth -1
				 */
	u16  words76_79[4];	/* reserved words 76-79 */
	u16  major_rev_num;	/* (word 80) */
	u16  minor_rev_num;	/* (word 81) */
	u16  command_set_1;	/* (word 82) supported
				 * 15:	Obsolete
				 * 14:	NOP command
				 * 13:	READ_BUFFER
				 * 12:	WRITE_BUFFER
				 * 11:	Obsolete
				 * 10:	Host Protected Area
				 *  9:	DEVICE Reset
				 *  8:	SERVICE Interrupt
				 *  7:	Release Interrupt
				 *  6:	look-ahead
				 *  5:	write cache
				 *  4:	PACKET Command
				 *  3:	Power Management Feature Set
				 *  2:	Removable Feature Set
				 *  1:	Security Feature Set
				 *  0:	SMART Feature Set
				 */
	u16  command_set_2;	/* (word 83)
				 * 15:	Shall be ZERO
				 * 14:	Shall be ONE
				 * 13:	FLUSH CACHE EXT
				 * 12:	FLUSH CACHE
				 * 11:	Device Configuration Overlay
				 * 10:	48-bit Address Feature Set
				 *  9:	Automatic Acoustic Management
				 *  8:	SET MAX security
				 *  7:	reserved 1407DT PARTIES
				 *  6:	SetF sub-command Power-Up
				 *  5:	Power-Up in Standby Feature Set
				 *  4:	Removable Media Notification
				 *  3:	APM Feature Set
				 *  2:	CFA Feature Set
				 *  1:	READ/WRITE DMA QUEUED
				 *  0:	Download MicroCode
				 */
	u16  cfsse;		/* (word 84)
				 * cmd set-feature supported extensions
				 * 15:	Shall be ZERO
				 * 14:	Shall be ONE
				 * 13:3	reserved
				 *  2:	Media Serial Number Valid
				 *  1:	SMART selt-test supported
				 *  0:	SMART error logging
				 */
	u16  cfs_enable_1;	/* (word 85)
				 * command set-feature enabled
				 * 15:	Obsolete
				 * 14:	NOP command
				 * 13:	READ_BUFFER
				 * 12:	WRITE_BUFFER
				 * 11:	Obsolete
				 * 10:	Host Protected Area
				 *  9:	DEVICE Reset
				 *  8:	SERVICE Interrupt
				 *  7:	Release Interrupt
				 *  6:	look-ahead
				 *  5:	write cache
				 *  4:	PACKET Command
				 *  3:	Power Management Feature Set
				 *  2:	Removable Feature Set
				 *  1:	Security Feature Set
				 *  0:	SMART Feature Set
				 */
	u16  cfs_enable_2;	/* (word 86)
				 * command set-feature enabled
				 * 15:	Shall be ZERO
				 * 14:	Shall be ONE
				 * 13:	FLUSH CACHE EXT
				 * 12:	FLUSH CACHE
				 * 11:	Device Configuration Overlay
				 * 10:	48-bit Address Feature Set
				 *  9:	Automatic Acoustic Management
				 *  8:	SET MAX security
				 *  7:	reserved 1407DT PARTIES
				 *  6:	SetF sub-command Power-Up
				 *  5:	Power-Up in Standby Feature Set
				 *  4:	Removable Media Notification
				 *  3:	APM Feature Set
				 *  2:	CFA Feature Set
				 *  1:	READ/WRITE DMA QUEUED
				 *  0:	Download MicroCode
				 */
	u16  csf_default;	/* (word 87)
				 * command set-feature default
				 * 15:	Shall be ZERO
				 * 14:	Shall be ONE
				 * 13:3	reserved
				 *  2:	Media Serial Number Valid
				 *  1:	SMART selt-test supported
				 *  0:	SMART error logging
				 */
	u16  dma_ultra;	/* (word 88) */
	u16	word89;		/* reserved (word 89) */
	u16	word90;		/* reserved (word 90) */
	u16	CurAPMvalues;	/* current APM values */
	u16	word92;		/* reserved (word 92) */
	u16	hw_config;	/* hardware config (word 93)
				 * 15:
				 * 14:
				 * 13:
				 * 12:
				 * 11:
				 * 10:
				 *  9:
				 *  8:
				 *  7:
				 *  6:
				 *  5:
				 *  4:
				 *  3:
				 *  2:
				 *  1:
				 *  0:
				 */
	u16	acoustic;	/* (word 94)
				 * 15:8	Vendor's recommended value
				 *  7:0	current value
				 */
	u16	words95_99[5];	/* reserved words 95-99 */
	u64	lba_capacity_2; /* 48-bit total number of sectors */
	u16	words104_125[22];/* reserved words 104-125 */
	u16	last_lun;	/* (word 126) */
	u16	word127;	/* (word 127) Feature Set
				 * Removable Media Notification
				 * 15:2	reserved
				 *  1:0	00 = not supported
				 *	01 = supported
				 *	10 = reserved
				 *	11 = reserved
				 */
	u16	dlf;		/* (word 128)
				 * device lock function
				 * 15:9	reserved
				 *  8	security level 1:max 0:high
				 *  7:6	reserved
				 *  5	enhanced erase
				 *  4	expire
				 *  3	frozen
				 *  2	locked
				 *  1	en/disabled
				 *  0	capability
				 */
	u16  csfo;		/*  (word 129)
				 * current set features options
				 * 15:4	reserved
				 *  3:	auto reassign
				 *  2:	reverting
				 *  1:	read-look-ahead
				 *  0:	write cache
				 */
	u16	words130_155[26];/* reserved vendor words 130-155 */
	u16	word156;	/* reserved vendor word 156 */
	u16	words157_159[3];/* reserved vendor words 157-159 */
	u16	cfa_power;	/* (word 160) CFA Power Mode
				 * 15 word 160 supported
				 * 14 reserved
				 * 13
				 * 12
				 * 11:0
				 */
	u16	words161_175[14];/* Reserved for CFA */
	u16	words176_205[31];/* Current Media Serial Number */
	u16	words206_254[48];/* reserved words 206-254 */
	u16	integrity_word;	/* (word 255)
				 * 15:8 Checksum
				 *  7:0 Signature
				 */
} __attribute__((packed));

typedef enum {
	ata_type_unknown = 123456,
	ata_type_disk = 7891011,

	ata_type_cdrom = 5,
	ata_type_floppy = 0,
	ata_type_optical = 7,
	ata_type_tape = 1,
} ata_device_t;

/*
 * Standard (generic) timings for PIO modes, from ATA2 specification.
 * These timings are for access to the IDE data port register *only*.
 * Some drives may specify a mode, while also specifying a different
 * value for cycle_time (from drive identification data).
 */
struct ata_pio_timings {
	int	setup_time;	/* Address setup (ns) minimum */
	int	active_time;	/* Active pulse (ns) minimum */
	int	cycle_time;	/* Cycle time (ns) minimum = (setup + active + recovery) */
};

struct ata_pio_data {
	u8 pio_mode;
	u8 use_iordy;
	u8 overridden;
	u8 blacklisted;
	unsigned int cycle_time;
};

struct pci_dev;
struct ata_device;
struct ata_channel;
struct ata_host;

/*
 * the ATA register values and other metadata we need
 * to tell hardware about an ATA command.
 */
struct ata_cmd {
	u32 flags;		/* see ATA_CMD_FL_xxx below */

	u8 command;		/* device command */
	u8 devctl;		/* control */
	u8 device;		/* device select */

	u8 data;		/* standard registers */
	u8 nsector;
	u8 lba_low;
	u8 lba_mid;
	u8 lba_high;
	u8 features;

	u8 hob_data;		/* HOB or hobfile. I wanna call */
	u8 hob_nsector;		/* it "bobfile", I wonder if they would */
	u8 hob_lba_low;		/* change the standards just for me. */
	u8 hob_lba_mid;
	u8 hob_lba_high;
	u8 hob_error;

	/* ... and the compiler pads a bit */
};

struct ata_ports {
	unsigned long lba_high;		/* LBA high */
	unsigned long lba_mid;		/* LBA mid */
	unsigned long lba_low;		/* LBA low */
	unsigned long device;		/* device */
	unsigned long nsector;		/* sector count */
	unsigned long status;		/* status (RO) */
	unsigned long command;		/* command (WO) */
	unsigned long error;		/* error (RO) */
	unsigned long features;		/* features (WO) */
	unsigned long data;		/* data in/out */
	unsigned long altstatus;	/* alternate status (RO) */
	unsigned long devctl;		/* device control (WO) */
};

struct ata_device {
	struct ata_ports ports;
	unsigned int device;		/* 0==dev0, ATA_DEV1==dev1 */
	struct list_head list_node;
	struct ata_channel *chan;
	ata_device_t dev_type;
	unsigned long flags;
	struct ata_id_page id;
	struct pci_dev *pdev;
	spinlock_t *lock;
	u64 capacity;			/* disk size, in number of sectors */
	u16 cyl;			/* 'cyl' part of C/H/S */
	u16 head;			/* 'head' part of C/H/S */
	u16 sect;			/* 'sectors' part of C/H/S */

	struct storage_dev sd;

	void (*cmd_prep) (struct ata_device *ad, struct ata_cmd *cmd);
	void (*set_pio) (struct ata_device *dev, u8 pio_mode);
};

struct ata_channel {
	unsigned int id;		/* channel number, 0 or 1, typically */
	struct ata_ports ports;
	struct ata_host *host;
	struct list_head dev_list;
	unsigned long flags;
	spinlock_t *lock;
	void *lock_base;
	unsigned int irq;
};

struct ata_host_info {
	unsigned int lock_strategy;
	unsigned int n_chan;
	unsigned long irq_flags;
	void (*cmd_prep) (struct ata_device *ad, struct ata_cmd *cmd);
	void (*set_pio) (struct ata_device *dev, u8 pio_mode);
};

struct ata_host {
	struct ata_host_info info;
	struct storage_host sh;

	struct ata_channel chan[0];
};

struct ata_class {
	const char *name;
	ata_device_t dev_type;
	struct list_head list_node;
	int (*add) (struct ata_device *dev);
	void (*remove) (struct ata_device *dev);
};

extern struct ata_cmd ata_cmd_prep[];
extern void ata_cmd_write_pio (struct ata_device *ad, struct ata_cmd *cmd);
extern int ata_cmd_wait (struct ata_device *dev);

extern int ata_host_register (struct ata_host *host);
extern void ata_host_unregister (struct ata_host *host);
extern struct ata_host *ata_host_alloc (struct ata_host_info *host_info);
extern void ata_host_free(struct ata_host *host);

extern int ata_chan_init (struct ata_channel *chan);
extern void ata_chan_shutdown (struct ata_channel *chan);

extern struct ata_ports ata_std_ports[ATA_STD_NCHAN];
extern void ata_pci_legacy_fixup (struct pci_dev *pdev);

extern u8 ata_get_best_pio_mode (struct ata_device *dev, u8 mode_wanted,
			  u8 max_mode, struct ata_pio_data *d);

extern int ata_class_register(struct ata_class *cls);
extern void ata_class_unregister(struct ata_class *cls);

#endif /* __LINUX_ATA_H__ */
